/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.api.abstractElmt;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Document;
import org.jdom.Namespace;
import org.ow2.easywsdl.schema.api.ComplexType;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.SimpleType;
import org.ow2.easywsdl.schema.api.absItf.AbsItfElement;
import org.ow2.easywsdl.schema.api.absItf.AbsItfType;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.FormChoice;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Group.ParticleItem;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public abstract class AbstractElementImpl<E, T extends AbsItfType> extends
AbstractSchemaElementImpl<E> implements AbsItfElement<T> {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;


	/**
	 * the type of element
	 */
	protected T type;

	/**
	 * referenced element
	 */
	protected AbsItfElement referencedElement;

	/**
	 *
	 * @param model
	 * @param parent
	 */
	public AbstractElementImpl(final E model, final AbstractSchemaElementImpl parent) {
		super(model, parent);
		this.referencedElement = null;
	}

	public T getType() {
		return this.type;
	}

	public void setType(T type) {
		this.type = type;
	}

	@SuppressWarnings("unchecked")
	public String printXml(final Object value) throws SchemaException {
		String res = null;
		if (this.type != null) {
			if (this.type instanceof SimpleType) {
				res = ((AbstractTypeImpl) this.type).printXml(this, value);
			} else if (this.type instanceof ComplexType) {
				final List<String> newValue = new ArrayList<String>();
				if (value instanceof String) {
					newValue.add((String) value);
				} else if (value instanceof List) {
					newValue.addAll((List<String>) value);
				}
				res = ((AbstractTypeImpl) this.type).printXml(this, newValue);
			}
		}
		return res;
	}

	public AbstractSchemaElementImpl getParent() {
		return this.parent;
	}

	@SuppressWarnings("unchecked")
	public org.jdom.Element generateElement(Object values) throws SchemaException {
		if(values == null) {
			values = "?";
		}

		// create element
		final org.jdom.Element eOut = new org.jdom.Element(this.getQName().getLocalPart());

		if(this.getSchema().getElementFormDefault() == FormChoice.QUALIFIED) {
			eOut.setNamespace(org.jdom.Namespace.getNamespace(this.getQName().getPrefix(), this
					.getQName().getNamespaceURI()));
		} 

		// add type attribute on element
		if (((AbstractElementImpl) this).getType() != null) {
			String type = null;
			// if not a anonymous type
			if (((AbstractElementImpl) this).getType().getQName() != null) {
				String prefix = ((AbstractElementImpl) this).getParent().getSchema().getAllNamespaces().getPrefix(
						((AbstractElementImpl) this).getType().getQName().getNamespaceURI());
				if (prefix != null) {
					type = prefix 
					+ ":"
					+ ((AbstractElementImpl) this).getType().getQName().getLocalPart();
					Namespace typeNs = Namespace.getNamespace(prefix, ((AbstractElementImpl) this).getParent().getSchema().getAllNamespaces().getNamespaceURI(prefix));
					eOut.addNamespaceDeclaration(typeNs);
					eOut.setAttribute("type", type);
				} else {
					type = ((AbstractElementImpl) this).getType().getQName().getLocalPart();
					eOut.setAttribute("type", type);
				}
			}
		}

		if(!this.isNillable()) {
			// if complextype, generate under element
			if (((AbstractElementImpl) this).getType() instanceof ComplexType) {
				if ((((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) this)
						.getType()).getModel()) != null) {

					List<ParticleItem> items = null;

					if ((((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) this)
							.getType()).getModel()).getSequence() != null) {
						// if sequence
						items = (((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) this)
								.getType()).getModel()).getSequence().getParticleItems();
					} else if ((((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) this)
							.getType()).getModel()).getAll() != null) {
						// if all
						items = (((org.ow2.easywsdl.schema.impl.ComplexTypeImpl) ((AbstractElementImpl) this)
								.getType()).getModel()).getAll().getParticleItems();
					}

					if (items != null) {
						for (final ParticleItem item : items) {

							// is element
							if (item.getItemElement() != null) {
								if (item.getItemElement().getRef() == null) {
									// is an element
									final AbsItfElement e = new org.ow2.easywsdl.schema.impl.ElementImpl(
											item.getItemElement(), ((AbstractElementImpl) this)
											.getParent());
									eOut.addContent(((AbstractElementImpl) e).generateElement(this
											.nextValue(values)).detach());
								} else {
									// is a ref on an element
									final AbsItfElement e = ((AbstractElementImpl) this).getParent().getSchema()
									.getElement(item.getItemElement().getRef());
									eOut.addContent(((AbstractElementImpl) e).generateElement(this
											.nextValue(values)).detach());
								}
							}
						}
					}

					// handle attribute
					for (final org.ow2.easywsdl.schema.api.Attribute attr : ((ComplexType) ((AbstractElementImpl) this)
							.getType()).getAttributes()) {
						final org.jdom.Attribute a = ((AbstractAttributeImpl) attr).generateAttribute(
								this, this.nextValue(values));
						if(a != null) {

							eOut.setAttribute(a);
						}
					}

				}
			} else {
				// add value on the simpletype
				final String val = this.nextValue(values);
				if (val != null) {
					eOut.setText(val);
				}
			}
		}
		Document doc = new Document(eOut); 
		return doc.getRootElement();
	}

	@SuppressWarnings("unchecked")
	private String nextValue(final Object values) {
		String res = null;
		if (values instanceof List) {
			res = (String) ((List) values).get(0);
		} else if (values instanceof String) {
			res = (String) values;
		}
		return res;
	}

	@Override
	public String toString() {
		return this.getQName().toString();
	}
}
