/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.api.abstractElmt;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.transform.dom.DOMSource;

import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.SchemaReader;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants;
import org.ow2.easywsdl.schema.api.absItf.AbsItfInclude;
import org.ow2.easywsdl.schema.api.absItf.AbsItfSchema;
import org.ow2.easywsdl.schema.util.SourceHelper;
import org.ow2.easywsdl.schema.util.Util;
import org.xml.sax.InputSource;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public abstract class AbstractIncludeImpl<E, S extends AbsItfSchema> extends
		AbstractSchemaElementImpl<E> implements AbsItfInclude<S> {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(AbstractIncludeImpl.class
			.getName());

	/**
	 * the parent
	 */
	protected S schema;

	/**
	 * Default constructor
	 * 
	 * @param model
	 *            the model
	 * @param parent
	 *            the parent description
	 */
	@SuppressWarnings("unchecked")
	public AbstractIncludeImpl(final URI baseURI, final E model,
			final Map<FeatureConstants, Object> features,
			final Map<URI, DOMSource> imports) throws SchemaException {
		super(model, null);

		String location = this.getLocationURI();

		if ((!Util.isURL(location)) && (baseURI != null)) {
			location = baseURI.getPath() + location;
		}

		if (features != null) {
			final Object obj = features.get(FeatureConstants.IMPORT_DOCUMENTS);
			if ((Boolean) obj) {
				this.retrieveInclude(location, features, imports);
			}
		}

		if (this.schema != null) {
			((AbstractSchemaImpl) this.schema).setFeatures(features);
		}

	}

	/**
	 * Default constructor
	 * 
	 * @param model
	 *            the model
	 * @param parent
	 *            the parent description
	 */
	@SuppressWarnings("unchecked")
	public AbstractIncludeImpl(final E model,
			final AbstractSchemaElementImpl parent,
			final Map<URI, DOMSource> imports) throws SchemaException {
		super(model, parent);

		String location = this.getLocationURI();

		if (((location != null)) && (!Util.isURL(location))
				&& (parent.getSchema().getDocumentBaseURI() != null)) {
			location = parent.getSchema().getDocumentBaseURI().getPath()
					+ location;
		}

		if (this.parent != null) {
			if (((AbstractSchemaImpl) this.parent).getFeatures() != null) {
				if ((((AbstractSchemaImpl) this.parent).getFeatures().get(
						FeatureConstants.IMPORT_DOCUMENTS) != null)
						&& ((Boolean) ((AbstractSchemaImpl) this.parent)
								.getFeatures().get(
										FeatureConstants.IMPORT_DOCUMENTS))) {
					this.retrieveInclude(location,
							((AbstractSchemaImpl) this.parent).getFeatures(),
							imports);
				}
			}
		}

		if ((parent != null) && (this.schema != null)) {
			((AbstractSchemaImpl) this.schema)
					.setFeatures(((AbstractSchemaImpl) this.parent)
							.getFeatures());
		}
	}

	private void retrieveInclude(final String location,
			final Map<FeatureConstants, Object> features,
			final Map<URI, DOMSource> imports) throws SchemaException {
		InputStream is = null;

		if (location != null) {

			try {
				// final URI loc = new URI(location);
				//
				// final File f = new File(location);
				// if (f.exists()) {
				// is = new FileInputStream(f);
				// } else {
				// is = loc.toURL().openStream();
				// }

				// Create schema reader
				final SchemaReader reader = new org.ow2.easywsdl.schema.impl.SchemaReaderImpl();

				URI loc;
				// Retrieve Source
				// Do not try to download a schema already present in classpath
				// (xml.xsd)
				if ("xml.xsd".equals(location.substring(location
						.lastIndexOf("/") + 1))) {
					final URL url = getClass().getClassLoader().getResource(
							"schema/xml.xsd");
					loc = new URI(url.toString());
				} else {
					loc = new URI(location);
				}

				DOMSource source = null;

				if (imports != null && imports.containsKey(loc)) {
					// Search in imports Map
					source = imports.get(loc);
				} else {
					// Try to download it
					final File f = new File(location);
					if (f.exists()) {
						is = new FileInputStream(f);
					} else {
						is = loc.toURL().openStream();
					}
					source = SourceHelper
							.convertInputSource2DOMSource(new InputSource(is));
				}

				((AbstractSchemaReaderImpl) reader).setFeatures(features);
				this.schema = (S) reader.readSchema(loc, source, imports);

				/*
				 * } catch (final MalformedURLException e) { // do nothing //
				 * the document is not included in imported document
				 * AbstractIncludeImpl
				 * .log.warning("the imported document cannot be import at: " +
				 * location + " => " + e.getMessage()); } catch (final
				 * URISyntaxException e) { // do nothing // the document is not
				 * included in imported document
				 * AbstractIncludeImpl.log.warning(
				 * "the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); } catch (final UnknownHostException
				 * e) { // do nothing // the document is not included in
				 * imported documentAbstractIncludeImpl.log.warning(
				 * "the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); } catch (final ConnectException e)
				 * { // do nothing // the document is not included in imported
				 * documentAbstractIncludeImpl.log.warning(
				 * "the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); } catch (final
				 * IllegalArgumentException e) { // do nothing // the document
				 * is not included in imported document
				 * AbstractIncludeImpl.log.warning
				 * ("the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); } catch (final IOException e) { //
				 * do nothing // the document is not included in imported
				 * documentAbstractIncludeImpl.log.warning(
				 * "the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); } catch (final SchemaException e) {
				 * // do nothing // the document is not included in imported
				 * document e.printStackTrace();
				 * AbstractIncludeImpl.log.warning(
				 * "the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); } catch (final XmlException e) { //
				 * do nothing // the document is not included in imported
				 * document e.printStackTrace();
				 * AbstractIncludeImpl.log.warning(
				 * "the imported document cannot be import at: " + location +
				 * " => " + e.getMessage()); }
				 */
			} catch (final Exception e) {
				throw new SchemaException(
						"the imported document cannot be import at: "
								+ location, e);
			}
		}
	}

	/**
	 * @return the parent
	 */
	public S getSchema() {
		return this.schema;
	}

	public void setSchema(final S schema) {
		this.schema = schema;
	}

	/**
	 * @return the parent parent
	 */
	public S getParentSchema() {
		return (S) this.parent.getSchema();
	}

}
