/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.api.abstractElmt;

import java.util.ArrayList;
import java.util.List;

import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.absItf.AbsItfAttribute;
import org.ow2.easywsdl.schema.api.absItf.AbsItfElement;
import org.ow2.easywsdl.schema.api.absItf.AbsItfType;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Annotated;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public abstract class AbstractTypeImpl<E extends Annotated, V, A extends AbsItfAttribute> extends
        AbstractSchemaElementImpl<E> implements AbsItfType<A> {

    /**
	 *
	 */
    private static final long serialVersionUID = 1L;



    /**
     * the outputter
     */
    protected XMLOutputter outputter = new XMLOutputter();

    /**
     * the list of attributes
     */
    protected List<A> attributes;

    /**
     *
     * @param model
     * @param parent
     */
    public AbstractTypeImpl(final E model, final AbstractSchemaElementImpl parent) {
        super(model, parent);
        this.outputter.setFormat(Format.getPrettyFormat());
    }

    public abstract String printXml(AbsItfElement e, V value) throws SchemaException;

    /*
     * Attributes methods
     */

    /*
     * (non-Javadoc)
     *
     * @see org.ow2.easywsdl.schema.api.Schema#getAttributes()
     */
    public List<A> getAttributes() {
        if (this.attributes == null) {
            this.attributes = new ArrayList<A>();
        }
        return this.attributes;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.ow2.easywsdl.schema.api.Schema#getAttribute(javax.xml.namespace
     * .QName)
     */
    public A getAttribute(final String element) {
        A res = null;
        if (this.attributes == null) {
            this.attributes = new ArrayList<A>();
        }
        for (final A e : this.attributes) {
            if (e.getName().equals(element)) {
                res = e;
                break;
            }
        }
        return res;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.ow2.easywsdl.schema.api.Schema#addAttribute(com.ebmwebsourcing
     * .commons.schema.api.Attribute)
     */
    public void addAttribute(final A elmt) {
        if (this.attributes == null) {
            this.attributes = new ArrayList<A>();
        }
        this.attributes.add(elmt);
    }

    @Override
    public String toString() {
        String res = "anonymous type";
        if (this.getQName() != null) {
            res = this.getQName().toString();
        }
        return res;
    }
}
