/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.easywsdl.schema.api.Attribute;
import org.ow2.easywsdl.schema.api.ComplexContent;
import org.ow2.easywsdl.schema.api.ComplexType;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.Sequence;
import org.ow2.easywsdl.schema.api.absItf.AbsItfElement;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractComplexTypeImpl;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractElementImpl;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaElementImpl;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ExplicitGroup;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexType.AttributeOrAttributeGroupItem;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class ComplexTypeImpl
        extends
        AbstractComplexTypeImpl<org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexType, List<String>, Attribute, Sequence, ComplexContent>
        implements ComplexType {

    /**
	 *
	 */
    private static final long serialVersionUID = 1L;

    public ComplexTypeImpl(
            final org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexType model,
            final AbstractSchemaElementImpl parent) {
        super(model, parent);

        // get attributes associated to the elements
        for (final AttributeOrAttributeGroupItem attr : model.getAttributeOrAttributeGroupItems()) {
            if (attr.getItemAttribute() != null) {
                final Attribute a = new org.ow2.easywsdl.schema.impl.AttributeImpl(attr
                        .getItemAttribute(), this);
                this.addAttribute(a);
            }
        }

        // get the sequence
        if(this.model.getSequence() != null) {
        	this.sequence = new SequenceImpl(this.model.getSequence(), this);
        }
        
    }

    public QName getQName() {
        QName res = null;
        if ((this.getModel()).getName() != null) {
            res = new QName(this.parent.getSchema().getTargetNamespace(), (this.getModel()).getName());
        }
        return res;
    }

    public void setQName(QName name) {
    	this.model.setName(name.getLocalPart());
    }

	@Override
    public String printXml(final AbsItfElement e, final List<String> values) throws SchemaException {
        final org.jdom.Element eOut = ((AbstractElementImpl) e).generateElement(values);
        return this.outputter.outputString(eOut);
    }
    

    @Override
	public void setSequence(Sequence sequence) {
		super.setSequence(sequence);
		this.model.setSequence((ExplicitGroup) ((AbstractSchemaElementImpl)this.sequence).getModel());
	}


	public Sequence createSequence() {
		return new SequenceImpl(new ExplicitGroup(), this);
	}
	
	

	@Override
	public void setComplexContent(ComplexContent complexContent) {
		super.setComplexContent(complexContent);
		this.model.setComplexContent((org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexContent) ((AbstractSchemaElementImpl)this.complexContent).getModel());
	}

	public ComplexContent createComplexContent() {
		return new ComplexContentImpl(new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexContent(), this);
	}

}
