/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.impl;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Map;

import javax.xml.transform.dom.DOMSource;

import org.apache.commons.lang.NotImplementedException;
import org.ow2.easywsdl.schema.api.Attribute;
import org.ow2.easywsdl.schema.api.Element;
import org.ow2.easywsdl.schema.api.Import;
import org.ow2.easywsdl.schema.api.Include;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.SimpleType;
import org.ow2.easywsdl.schema.api.Type;
import org.ow2.easywsdl.schema.api.SchemaReader.FeatureConstants;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaElementImpl;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaImpl;
import org.ow2.easywsdl.schema.api.extensions.NamespaceContextImpl;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Annotated;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.FormChoice;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.OpenAttrs;
import org.ow2.easywsdl.schema.org.w3._2001.xmlschema.TopLevelElement;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class SchemaImpl
extends
AbstractSchemaImpl<org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema, Type, Element, Attribute, Include, Import>
implements Schema {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	public SchemaImpl(final URI baseURI,
			final org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema schema,
			final org.jdom.Document doc, final Map<FeatureConstants, Object> features, final Map<URI, DOMSource> imports)
	throws SchemaException {
		super(baseURI, schema, doc);

		this.features = features;
		this.model = schema;
		
		for (final OpenAttrs item : this.model.getIncludeOrImportOrRedefine()) {
			// get imports
			if (item instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Import) {
				final Import impt = new org.ow2.easywsdl.schema.impl.ImportImpl(
						(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Import) item,
						this, imports);
				this.imports.add(impt);
			}

			// get includes
			if (item instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Include) {
				final Include incl = new org.ow2.easywsdl.schema.impl.IncludeImpl(
						(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Include) item,
						this, imports);
				this.includes.add(incl);
			}
		}
		this.addImportElementsInAllList();
		this.addIncludeElementsInAllList();

		for (final Annotated item : this.model.getSimpleTypeOrComplexTypeOrGroup()) {

			// get simple type
			if (item instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.SimpleType) {
				final SimpleType st = new org.ow2.easywsdl.schema.impl.SimpleTypeImpl(
						(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.SimpleType) item,
						this);
				this.types.add(st);
			}

			// get complex type
			if (item instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexType) {
				final ComplexTypeImpl ct = new org.ow2.easywsdl.schema.impl.ComplexTypeImpl(
						(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.ComplexType) item,
						this);
				this.types.add(ct);
			}
		}

		for (final Annotated item : this.model.getSimpleTypeOrComplexTypeOrGroup()) {

			// get elements
			if (item instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Element) {
				final ElementImpl e = new org.ow2.easywsdl.schema.impl.ElementImpl(
						(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Element) item,
						this);
				this.elements.add(e);
			}

			// get attributes
			if (item instanceof org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Attribute) {
				final AttributeImpl a = new org.ow2.easywsdl.schema.impl.AttributeImpl(
						(org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Attribute) item,
						this);
				this.attributes.add(a);
			}
		}

	}


	public SchemaImpl() {
		try {
			this.model = new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema();
			this.baseURI = new URI(".");
			((NamespaceContextImpl)this.namespaceContext).addNamespace("xsd", Constants.SCHEMA_NAMESPACE);
		} catch (URISyntaxException e) {
			// do nothing
			e.printStackTrace();
		}
	}

	public String getTargetNamespace() {
		return this.model.getTargetNamespace();
	}
	
	
	public FormChoice getElementFormDefault() {
		return this.model.getElementFormDefault();
	}

	public FormChoice getAttributeFormDefault() {
		return this.model.getAttributeFormDefault();
	}
	
	public String getLang() {
		return this.model.getLang();
	}
	
	public String getVersion() {
		return this.model.getVersion();
	}
	
	public List<String> getBlockDefault() {
		return this.model.getBlockDefault();
	}
	
	public List<String> getFinalDefault() {
		return this.model.getFinalDefault();
	}
	

	@Override
	public void addAttribute(Attribute att) {
		super.addAttribute(att);
		this.model.getSimpleTypeOrComplexTypeOrGroup().add((Annotated) ((AbstractSchemaElementImpl)att).getModel());

	}


	@Override
	public void addImport(Import impt) {
		super.addImport(impt);
		this.model.getIncludeOrImportOrRedefine().add((Annotated) ((AbstractSchemaElementImpl)impt).getModel());
	}


	@Override
	public void addInclude(Include incl) throws SchemaException {
		super.addInclude(incl);
		this.model.getIncludeOrImportOrRedefine().add((Annotated) ((AbstractSchemaElementImpl)incl).getModel());
	}


	@Override
	public void addElement(Element elmt) {
		super.addElement(elmt);
		this.model.getSimpleTypeOrComplexTypeOrGroup().add((Annotated) ((AbstractSchemaElementImpl)elmt).getModel());
	}


	@Override
	public void addType(Type type) {
		super.addType(type);
		this.model.getSimpleTypeOrComplexTypeOrGroup().add((Annotated) ((AbstractSchemaElementImpl)type).getModel());
	}

	public Import removeImport(final Import importDef) {
		throw new NotImplementedException();
	}

	public Include removeInclude(final Include includeDef) throws SchemaException {
		throw new NotImplementedException();
	}

	public Attribute createAttribute() {
		return new AttributeImpl(new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Attribute(), this);
	}

	public Type createComplexType() {
		return new ComplexTypeImpl(new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.TopLevelComplexType(), this);
	}

	public Element createElement() {
		return new ElementImpl(new TopLevelElement(), this);
	}

	public Type createSimpleType() {
		return new SimpleTypeImpl(new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.TopLevelSimpleType(), this);
	}


	public void setTargetNamespace(String tns) {
		this.model.setTargetNamespace(tns);
	}


	public Import createImport() throws SchemaException{
		return new ImportImpl(new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Import(), this, null);
	}


	public Include createInclude() throws SchemaException{
		return new IncludeImpl(new org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Include(), this, null);
	}

}
