/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import org.jdom.JDOMException;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.api.XmlException;
import org.ow2.easywsdl.schema.api.abstractElmt.AbstractSchemaReaderImpl;
import org.ow2.easywsdl.schema.util.JDomAnalyzer;
import org.ow2.easywsdl.schema.util.SourceHelper;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class SchemaReaderImpl extends AbstractSchemaReaderImpl implements
        org.ow2.easywsdl.schema.api.SchemaReader {

    /**
     * The JAXB unique unmarshaller.
     */
    private static Unmarshaller unmarshaller = null;

    /**
     * The exception raised during JAXB unique unmarshaller creation.
     */
    private static SchemaException unmarshallerCreationEx = null;

    /*
     * Private object initializations
     */
    static {
        try {

            SchemaJAXBContext.createContext();

            SchemaReaderImpl.unmarshaller = SchemaJAXBContext.getJaxbContext().createUnmarshaller();
            // unmarshaller.setSchema(parent);
        } catch (final JAXBException e) {
            final SchemaException jaxbContextCreationEx = new SchemaException(
                    "Failed to create jaxb context", e);
            SchemaReaderImpl.unmarshallerCreationEx = new SchemaException(
                    "Failed to create the JAXB unmarshaller", jaxbContextCreationEx);

        } catch (final SchemaException e) {
            final SchemaException jaxbContextCreationEx = new SchemaException(
                    "Failed to create jaxb context", e);
            SchemaReaderImpl.unmarshallerCreationEx = new SchemaException(
                    "Failed to create the JAXB unmarshaller", jaxbContextCreationEx);

        }
    }

    private static org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema convertStream2Schema(
            final Source schemaDescriptorStream) throws SchemaException {

        try {
            // TODO : Check if it is a Thread safe method
            final JAXBElement<org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema> schemaBinding = SchemaReaderImpl
                    .getUnMarshaller().unmarshal(schemaDescriptorStream,
                    		org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema.class);

            return schemaBinding.getValue();

        } catch (final JAXBException e) {
            throw new SchemaException(
                    "Failed to build Java bindings from SchemaImpl descriptor XML document", e);
        }
    }

    /**
     * Get the unmarshaller instance.
     * 
     * @return the unmarshaller instance
     * @throws SchemaException
     *             The exception raised during the unmarshaller creation.
     */
    public static Unmarshaller getUnMarshaller() throws SchemaException {

        if (SchemaReaderImpl.unmarshallerCreationEx != null) {
            throw SchemaReaderImpl.unmarshallerCreationEx;
        }

        return SchemaReaderImpl.unmarshaller;
    }

    /**
     * @return the jaxbContext
     */
    public static JAXBContext getJaxbContext() {
        return SchemaJAXBContext.getJaxbContext();
    }

    public Schema readSchema(final URI schemaURI) throws SchemaException {
        Schema desc = null;
        try {
            InputStream input = null;
            File f = null;
            if(schemaURI.getPath() != null) {
            	f = new File(schemaURI.getPath());
            }
            if (f != null) {
                input = new FileInputStream(f);
            } else {
                input = schemaURI.toURL().openStream();
            }
            desc = this.readSchema(schemaURI, SourceHelper
                    .convertInputSource2DOMSource(new InputSource(input)));
        } catch (final MalformedURLException e) {
            throw new SchemaException("Can not get wsdl at: " + schemaURI, e);
        } catch (final IOException e) {
            throw new SchemaException("Can not get wsdl at: " + schemaURI, e);
        } catch (final SchemaException e) {
            throw new SchemaException("Can not get wsdl at: " + schemaURI, e);
        } catch (final XmlException e) {
            throw new SchemaException("Can not get wsdl at: " + schemaURI, e);
        }
        return desc;
    }

    public Schema readSchema(final Document schemaDocument, Map<URI, Document> imports) throws SchemaException {
        Schema desc = null;
        try {
			// Convert DOM imports to DOMSource import
			Map<URI, DOMSource> importSources = new HashMap<URI, DOMSource>();
			if (imports != null && imports.size() > 0) {
				Set<URI> keys = imports.keySet();
				for (URI key : keys) {
					importSources.put(key, new DOMSource(imports.get(key)));
				}
			}
        	
			// Read Schema
            desc = this.readSchema(new URI(schemaDocument.getDocumentURI()), new DOMSource(
                    schemaDocument), importSources);
        } catch (final URISyntaxException e) {
            throw new SchemaException(e);
        }

        return desc;
    }

    public Schema readSchema(final URI schemaUri, final DOMSource input,
			Map<URI, DOMSource> imports) throws SchemaException {
        Schema desc = null;
        try {
            final DOMSource source1 = SourceHelper.cloneSource(input);
            final DOMSource source2 = SourceHelper.cloneSource(input);

            final org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema def = SchemaReaderImpl
                    .convertStream2Schema(source1);
            final JDomAnalyzer analyzer = new JDomAnalyzer(SourceHelper
                    .convertDOMSource2InputSource(source2));
            final org.jdom.Document doc = analyzer.getDocument();

            desc = new org.ow2.easywsdl.schema.impl.SchemaImpl(schemaUri, def, doc, this
                    .getFeatures(), imports);
        } catch (final JDOMException e) {
            throw new SchemaException(e);
        } catch (final XmlException e) {
            throw new SchemaException(e);
        }

        return desc;
    }

	public Schema readSchema(Document wsdlDocument) throws SchemaException {
		return this.readSchema(wsdlDocument, null);
	}

	public Schema readSchema(final URI wsdlURI, final DOMSource inputSource)
			throws SchemaException {
		return this.readSchema(wsdlURI, inputSource, null);
	}
}
