/**
 * easySchema - easyWSDL toolbox Platform.
 * Copyright (c) 2008,  eBM Websourcing
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.ow2.easywsdl.schema.impl;

import java.io.StringWriter;

import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.lang.NotImplementedException;
import org.ow2.easywsdl.schema.api.Schema;
import org.ow2.easywsdl.schema.api.SchemaException;
import org.ow2.easywsdl.schema.util.CustomPrefixMapper;
import org.w3c.dom.Document;

import com.sun.xml.bind.marshaller.NamespacePrefixMapper;


/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class SchemaWriterImpl extends SchemaJAXBContext implements
        org.ow2.easywsdl.schema.api.SchemaWriter {

    /**
     * The JAXB unique marshaller.
     */
    private static Marshaller marshaller = null;

    /**
     * The exception raised during JAXB unique marshaller creation.
     */
    private static SchemaException marshallerCreationEx = null;

    /*
     * Private object initializations
     */
    static {
        try {
            SchemaJAXBContext.createContext();
            SchemaWriterImpl.marshaller = SchemaJAXBContext.getJaxbContext().createMarshaller();
            // marshaller.setSchema(parent);
            
            NamespacePrefixMapper mapper = new CustomPrefixMapper();
            
            SchemaWriterImpl.marshaller.setProperty("com.sun.xml.bind.namespacePrefixMapper", mapper);

            
        } catch (final JAXBException e) {
            final SchemaException schemaCreationEx = new SchemaException(
                    "Failed to create context", e);

            SchemaWriterImpl.marshallerCreationEx = new SchemaException(
                    "Failed to create the JAXB marshaller", schemaCreationEx);

        } catch (final SchemaException e) {
            final SchemaException schemaCreationEx = new SchemaException(
                    "Failed to create context", e);

            SchemaWriterImpl.marshallerCreationEx = new SchemaException(
                    "Failed to create the JAXB marshaller", schemaCreationEx);

        }
    }

    @SuppressWarnings("unchecked")
    public static Document convertSchema2DOMElement(
            final org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema schemaDescriptor)
            throws SchemaException {
        Document doc = null;
        try {
            doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();

            // TODO : Check if it is a Thread safe method
            final JAXBElement element = new JAXBElement(new QName(Constants.SCHEMA_NAMESPACE,
                    Constants.SCHEMA_ROOT_TAG), schemaDescriptor.getClass(), schemaDescriptor);

            SchemaWriterImpl.marshaller.marshal(element, doc);
        } catch (final JAXBException ex) {
            throw new SchemaException(
                    "Failed to build XML binding from SchemaImpl descriptor Java classes", ex);
        } catch (final ParserConfigurationException ex) {
            throw new SchemaException(
                    "Failed to build XML binding from SchemaImpl descriptor Java classes", ex);

        }
        return doc;
    }

    @SuppressWarnings("unchecked")
    public static String convertSchema2String(
            final org.ow2.easywsdl.schema.org.w3._2001.xmlschema.Schema schemaDescriptor)
            throws SchemaException {

        try {
            final StringWriter stringWriter = new StringWriter();
            // TODO : Check if it is a Thread safe method

            final JAXBElement element = new JAXBElement(new QName(Constants.SCHEMA_NAMESPACE,
                    Constants.SCHEMA_ROOT_TAG), schemaDescriptor.getClass(), schemaDescriptor);

            SchemaWriterImpl.marshaller.marshal(element, stringWriter);

            return stringWriter.toString();
        } catch (final JAXBException e) {
            throw new SchemaException(
                    "Failed to build XML binding from Agreement descriptor Java classes", e);
        }
    }

    /**
     * Get the marshaller instance.
     *
     * @return the marshaller instance
     * @throws SchemaException
     *             The exception raised during the marshaller creation.
     */
    public static Marshaller getMarshaller() throws SchemaException {

        if (SchemaWriterImpl.marshallerCreationEx != null) {
            throw SchemaWriterImpl.marshallerCreationEx;
        }

        return SchemaWriterImpl.marshaller;
    }

    public Document getDocument(final Schema schemaDef) throws SchemaException {
        Document doc = null;
        if ((schemaDef != null)
                && (schemaDef instanceof org.ow2.easywsdl.schema.impl.SchemaImpl)) {
            try {
                doc = SchemaWriterImpl
                        .convertSchema2DOMElement(((org.ow2.easywsdl.schema.impl.SchemaImpl) schemaDef)
                                .getModel());
                doc.setDocumentURI(schemaDef.getDocumentBaseURI().getPath());
            } catch (final SchemaException e) {
                throw new SchemaException("Can not write wsdl description", e);
            }
        }
        return doc;
    }

    public boolean getFeature(final String name) throws IllegalArgumentException {
        throw new NotImplementedException();
    }

    public void setFeature(final String name, final boolean value) throws IllegalArgumentException {
        throw new NotImplementedException();
    }

    public String writeSchema(final Schema schemaDef) throws SchemaException {
        String res = null;
        if ((schemaDef != null)
                && (schemaDef instanceof org.ow2.easywsdl.schema.impl.SchemaImpl)) {
            try {
                res = SchemaWriterImpl
                        .convertSchema2String(((org.ow2.easywsdl.schema.impl.SchemaImpl) schemaDef)
                                .getModel());
            } catch (final SchemaException e) {
                throw new SchemaException("Can not write wsdl description", e);
            }
        }
        return res;
    }

}
